@@ -59,6 +59,7 @@
 #include "ardour/route.h"
 #include "ardour/route_group.h"
 #include "ardour/send.h"
+#include "ardour/selection.h"
 #include "ardour/session.h"
 #include "ardour/types.h"
 #include "ardour/user_bundle.h"
@@ -73,6 +74,7 @@
 
 #include "widgets/tooltips.h"
 
+#include "ardour_ui.h"
 #include "ardour_window.h"
 #include "automation_controller.h"
 #include "context_menu_helper.h"
@@ -411,8 +413,8 @@ MixerStrip::init ()
 
 	parameter_changed (X_("mixer-element-visibility"));
 	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &MixerStrip::parameter_changed));
-	Config->ParameterChanged.connect (_config_connection, MISSING_INVALIDATOR, boost::bind (&MixerStrip::parameter_changed, this, _1), gui_context());
-	_session->config.ParameterChanged.connect (_config_connection, MISSING_INVALIDATOR, boost::bind (&MixerStrip::parameter_changed, this, _1), gui_context());
+	 Config->ParameterChanged.connect (_config_connection, invalidator (*this), boost::bind (&MixerStrip::parameter_changed, this, _1), gui_context());
+	 _session->config.ParameterChanged.connect (_config_connection, invalidator (*this), boost::bind (&MixerStrip::parameter_changed, this, _1), gui_context());
 
 	//watch for mouse enter/exit so we can do some stuff
 	signal_enter_notify_event().connect (sigc::mem_fun(*this, &MixerStrip::mixer_strip_enter_event ));
@@ -500,20 +502,41 @@ MixerStrip::update_trim_control ()
 }
 
 void
-MixerStrip::trim_start_touch ()
+MixerStrip::trim_start_touch (int)
 {
 	assert (_route && _session);
-	if (route()->trim() && route()->trim()->active() && route()->n_inputs().n_audio() > 0) {
-		route()->trim()->gain_control ()->start_touch (timepos_t (_session->transport_sample()));
+
+	if (!route()->trim() || !route()->trim()->active() || route()->n_inputs().n_audio() <= 0) {
+		return;
 	}
+
+	std::shared_ptr<AutomationControl> control (route()->trim()->gain_control());
+
+	StripableList sl;
+	_session->selection ().get_stripables_for_op (sl, _route, &RouteGroup::is_gain);
+
+	_touch_control_group.reset (new GainControlGroup (TrimAutomation));
+	_touch_control_group->set_mode (ControlGroup::Relative);
+	_touch_control_group->fill_from_stripable_list (sl, control->parameter());
+
+	control->start_touch (timepos_t (_session->transport_sample()));
 }
 
 void
-MixerStrip::trim_end_touch ()
+MixerStrip::trim_end_touch (int)
 {
 	assert (_route && _session);
+
 	if (route()->trim() && route()->trim()->active() && route()->n_inputs().n_audio() > 0) {
-		route()->trim()->gain_control ()->stop_touch (timepos_t (_session->transport_sample()));
+
+		std::shared_ptr<AutomationControl> control (route()->trim()->gain_control());
+
+		control->stop_touch (timepos_t (_session->transport_sample()));
+
+		if (_touch_control_group) {
+			_touch_control_group->pop_all ();
+			_touch_control_group.reset ();
+		}
 	}
 }
 
@@ -1187,7 +1210,7 @@ MixerStrip::name_button_button_press (GdkEventButton* ev)
 			Gtkmm2ext::anchored_menu_popup(route_ops_menu, &name_button, "",
 			                               1, ev->time);
 		} else {
-			route_ops_menu->popup (3, ev->time);
+			route_ops_menu->popup (ev->button, ev->time);
 		}
 
 		return true;
@@ -1207,7 +1230,7 @@ MixerStrip::number_button_button_press (GdkEventButton* ev)
 	if (ev->button == 3) {
 		list_route_operations ();
 
-		route_ops_menu->popup (1, ev->time);
+		route_ops_menu->popup (ev->button, ev->time);
 
 		return true;
 	}
@@ -1475,24 +1498,24 @@ MixerStrip::meter_point_string (MeterPoint mp)
 	case Wide:
 		switch (mp) {
 		case MeterInput:
-			return _("In");
+			return S_("MeterWide|In");
 			break;
 
 		case MeterPreFader:
-			return _("Pre");
+			return S_("MeterWide|Pre");
 			break;
 
 		case MeterPostFader:
-			return _("Post");
+			return S_("MeterWide|Post");
 			break;
 
 		case MeterOutput:
-			return _("Out");
+			return S_("MeterWide|Out");
 			break;
 
 		case MeterCustom:
 		default:
-			return _("Custom");
+			return S_("MeterWide|Custom");
 			break;
 		}
 		break;
@@ -1671,8 +1694,8 @@ MixerStrip::set_button_names ()
 	switch (_width) {
 	case Wide:
 		mute_button->set_text (_("Mute"));
-		monitor_input_button->set_text (_("In"));
-		monitor_disk_button->set_text (_("Disk"));
+		monitor_input_button->set_text (S_("Monitor|In"));
+		monitor_disk_button->set_text (S_("Monitor|Disk"));
 		if (monitor_section_button) {
 			monitor_section_button->set_text (_("Mon"));
 		}
