@@ -80,6 +80,7 @@ namespace ARDOUR {
 		PBD::PropertyDescriptor<float> shift;
 		PBD::PropertyDescriptor<uint64_t> layering_index;
 		PBD::PropertyDescriptor<std::string> tags;
+		PBD::PropertyDescriptor<uint64_t> reg_group;
 		PBD::PropertyDescriptor<bool> contents;
 
 /* these properties are used as a convenience for announcing changes to state, but aren't stored as properties */
@@ -90,6 +91,43 @@ namespace ARDOUR {
 
 PBD::Signal2<void,std::shared_ptr<ARDOUR::RegionList>,const PropertyChange&> Region::RegionsPropertyChanged;
 
+/* these static values are used by Region Groups to assign a group-id across the scope of an operation that might span many function calls */
+uint64_t Region::_retained_group_id = 0;
+uint64_t Region::_next_group_id     = 0;
+
+std::map<uint64_t, uint64_t> Region::_operation_rgroup_map;
+Glib::Threads::Mutex         Region::_operation_rgroup_mutex;
+
+/* access the group-id for an operation on a region, honoring the existing region's group status */
+uint64_t
+Region::get_region_operation_group_id (uint64_t old_region_group, RegionOperationFlag flags) {
+	/* if the region was ungrouped, stay ungrouped */
+	if ((old_region_group == NoGroup) || (old_region_group == Explicit)) {
+		return old_region_group;
+	}
+
+	/* separate and preserve the Explicit flag: */
+	bool expl = (old_region_group & Explicit) == Explicit;
+
+	/* remove all flags */
+	old_region_group = (old_region_group >> 4) << 4;
+
+	/* apply flags to create a key, which will be used to recognize regions that belong in the same group */
+	uint64_t region_group_key = old_region_group | flags;
+
+	/* since the GUI is single-threaded, and it's hard/impossible to edit
+	 * during a rec-stop, this 'should' never be contentious
+	 */
+	Glib::Threads::Mutex::Lock lm (_operation_rgroup_mutex);
+
+	/* if a region group has not been assigned for this key, assign one */
+	if (_operation_rgroup_map.find (region_group_key) == _operation_rgroup_map.end ()) {
+		_operation_rgroup_map[region_group_key] = _next_group_id++;
+	}
+
+	return ((_operation_rgroup_map[region_group_key] << 4) | expl);
+}
+
 void
 Region::make_property_quarks ()
 {
@@ -147,6 +185,8 @@ Region::make_property_quarks ()
 	DEBUG_TRACE (DEBUG::Properties, string_compose ("quark for contents = %1\n",	Properties::contents.property_id));
 	Properties::time_domain.property_id = g_quark_from_static_string (X_("time_domain"));
 	DEBUG_TRACE (DEBUG::Properties, string_compose ("quark for time_domain = %1\n",	Properties::time_domain.property_id));
+	Properties::reg_group.property_id = g_quark_from_static_string (X_("rgroup"));
+	DEBUG_TRACE (DEBUG::Properties, string_compose ("quark for region_group = %1\n", Properties::reg_group.property_id));
 }
 
 void
@@ -177,6 +217,7 @@ Region::register_properties ()
 	add_property (_shift);
 	add_property (_layering_index);
 	add_property (_tags);
+	add_property (_reg_group);
 	add_property (_contents);
 }
 
@@ -208,6 +249,7 @@ Region::register_properties ()
 	, _shift (Properties::shift, 1.0) \
 	, _layering_index (Properties::layering_index, 0) \
 	, _tags (Properties::tags, "") \
+	, _reg_group (Properties::reg_group, 0) \
 	, _contents (Properties::contents, false)
 
 #define REGION_COPY_STATE(other) \
@@ -240,6 +282,7 @@ Region::register_properties ()
 	, _shift (Properties::shift, other->_shift) \
 	, _layering_index (Properties::layering_index, other->_layering_index) \
 	, _tags (Properties::tags, other->_tags) \
+	, _reg_group (Properties::reg_group, other->_reg_group) \
 	, _contents (Properties::contents, other->_contents)
 
 /* derived-from-derived constructor (no sources in constructor) */
@@ -507,7 +550,27 @@ Region::set_length_internal (timecnt_t const & len)
 	 * component of _length, and set_position() can only be used to set the
 	 * position component.
 	 */
+
 	_last_length = timecnt_t (_length.val().distance(), _last_length.position());
+
+	std::shared_ptr<Playlist> pl (playlist());
+
+	if (pl) {
+		Temporal::TimeDomain td (pl->time_domain());
+
+		/* Note: timecnt_t::time_domain() returns the domain for the
+		 * length component, *not* the position.
+		 */
+
+		if (td != len.time_domain()) {
+			timecnt_t l = _length.val();
+			l.set_time_domain (td);
+			_length = l;
+			return;
+		}
+	}
+	/* either no playlist or time domain for distance is not changing */
+
 	_length = timecnt_t (len.distance(), _length.val().position());
 }
 
@@ -608,7 +671,20 @@ Region::set_position_time_domain (Temporal::TimeDomain td)
 	timecnt_t t (length ().distance (), p);
 	_length = t;
 
-	/* ensure time-domain matches Datatype -- this may trigger in old broken sessions */
+	/* for a while, we allowed the time domain of _length to not match the
+	 * region time domain. This ought to be prevented as of August 7th 2023
+	 * or earlier, but let's not abort() out if asked to load sessions
+	 * where this happened. Note that for reasons described in the previous
+	 * comment, this could still cause issues during reading from disk.
+	 */
+
+	if  (_length.val().time_domain () != time_domain ()) {
+		_length.non_const_val().set_time_domain (time_domain());
+#ifndef NDEBUG
+		std::cerr << "Fixed up a" << (time_domain() == Temporal::AudioTime ? "n audio" : " music") << "-timed region called " << name() << std::endl;
+#endif
+	}
+
 	assert (_length.val().time_domain () == time_domain ());
 
 	send_change (Properties::time_domain);
@@ -689,6 +765,10 @@ Region::set_position_unchecked (timepos_t const & pos)
 void
 Region::set_position_internal (timepos_t const & pos)
 {
+	if (position() == pos) {
+		return;
+	}
+
 	/* We emit a change of Properties::length even if the position hasn't changed
 	 * (see Region::set_position), so we must always set this up so that
 	 * e.g. Playlist::notify_region_moved doesn't use an out-of-date last_position.
@@ -700,20 +780,41 @@ Region::set_position_internal (timepos_t const & pos)
 	 * position component.
 	 */
 
-	if (position() != pos) {
+	_last_length.set_position (position());
 
-		_last_length.set_position (position());
-		_length = timecnt_t (_length.val().distance(), pos);
+	std::shared_ptr<Playlist> pl (playlist());
 
-		/* check that the new _position wouldn't make the current
-		 * length impossible - if so, change the length.
-		 *
-		 * XXX is this the right thing to do?
+	if (pl) {
+		Temporal::TimeDomain td (pl->time_domain());
+
+		/* Note: timecnt_t::time_domain() returns the domain for the
+		 * length component, *not* the position.
 		 */
-		if (timepos_t::max (_length.val().time_domain()).earlier (_length) < position()) {
-			_last_length = _length;
-			_length = position().distance (timepos_t::max (position().time_domain()));
+
+		if (td != _length.val().position().time_domain()) {
+			timecnt_t l = _length.val();
+			l.set_position (pos);
+			l.set_time_domain (td);
+			_length = l;
+		} else {
+			/* time domain of position not changing */
+			_length = timecnt_t (_length.val().distance(), pos);
 		}
+
+	} else {
+		/* no playlist, so time domain is free to change */
+		_length = timecnt_t (_length.val().distance(), pos);
+	}
+
+
+	/* check that the new _position wouldn't make the current
+	 * length impossible - if so, change the length.
+	 *
+	 * XXX is this the right thing to do?
+	 */
+	if (timepos_t::max (_length.val().time_domain()).earlier (_length) < position()) {
+		_last_length = _length;
+		_length = position().distance (timepos_t::max (position().time_domain()));
 	}
 }
 
@@ -898,10 +999,13 @@ Region::cut_end (timepos_t const & new_endpoint)
 
 
 void
-Region::modify_front_unchecked (timepos_t const & new_position, bool reset_fade)
+Region::modify_front_unchecked (timepos_t const & npos, bool reset_fade)
 {
-	timepos_t last = end().decrement();
+	timepos_t last = nt_last();
 	timepos_t source_zero;
+	timepos_t new_position (npos);
+
+	new_position.set_time_domain (time_domain());
 
 	if (position() > start()) {
 		source_zero = source_position ();
@@ -981,10 +1085,14 @@ Region::trim_to (timepos_t const & position, timecnt_t const & length)
 }
 
 void
-Region::trim_to_internal (timepos_t const & pos, timecnt_t const & len)
+Region::trim_to_internal (timepos_t const & npos, timecnt_t const & nlen)
 {
-	timepos_t new_start (len.time_domain());
+	timepos_t pos (npos);
+	pos.set_time_domain (time_domain());
+	timecnt_t len (nlen);
+	len.set_time_domain (time_domain());
 
+	timepos_t new_start (time_domain());
 	timecnt_t const start_shift = position().distance (pos);
 
 	if (start_shift.is_positive()) {
@@ -2089,7 +2197,7 @@ Region::end() const
 	/* one day we might want to enforce _position, _start and _length (or
 	   some combination thereof) all being in the same time domain.
 	*/
-	return position() + _length.val();
+	return _length.val().end();
 }
 
 timepos_t
@@ -2117,7 +2225,13 @@ Region::source_beats_to_absolute_beats (Temporal::Beats beats) const
 Temporal::Beats
 Region::absolute_time_to_region_beats(timepos_t const & b) const
 {
-	 return (position().distance (b)).beats () + start().beats();;
+	 return (position().distance (b)).beats () + start().beats();
+}
+
+Temporal::timepos_t
+Region::absolute_time_to_region_time (timepos_t const & t) const
+{
+	return start() + position().distance (t);
 }
 
 Temporal::timepos_t
@@ -2193,13 +2307,42 @@ Region::time_domain() const
 }
 
 void
-Region::globally_change_time_domain (Temporal::TimeDomain from, Temporal::TimeDomain to)
+Region::start_domain_bounce (Temporal::DomainBounceInfo& cmd)
 {
-	assert (Temporal::domain_swap);
-
-	if (_length.val().time_domain() == from) {
-		timecnt_t& l (_length.non_const_val());
-		l.set_time_domain (to);
-		Temporal::domain_swap->add (l);
+	if (locked()) {
+		return;
 	}
+
+	/* recall that the _length member is a timecnt_t, and so holds both
+	 * position *and* length.
+	 */
+
+	if (_length.val().time_domain() != cmd.from) {
+		return;
+	}
+
+	timecnt_t& l (_length.non_const_val());
+
+	timecnt_t  saved (l);
+	saved.set_time_domain (cmd.to);
+
+	cmd.counts.insert (std::make_pair (&l, saved));
+}
+
+void
+Region::finish_domain_bounce (Temporal::DomainBounceInfo& cmd)
+{
+	clear_changes ();
+
+	Temporal::TimeDomainCntChanges::iterator tc = cmd.counts.find (&_length.non_const_val());
+	if (tc == cmd.counts.end()) {
+		/* must have already been in the correct time domain */
+		return;
+	}
+
+	/* switch domains back (but with modified TempoMap, presumably */
+	tc->second.set_time_domain (cmd.from);
+	_length = tc->second;
+
+	send_change (Properties::length);
 }
