@@ -341,7 +341,16 @@ VST3Plugin::possible_output () const
 bool
 VST3Plugin::has_editor () const
 {
-	return _plug->has_editor ();
+	VST3PI::RouteProcessorChangeBlock rpcb (_plug);
+
+	std::shared_ptr<VST3PluginInfo> nfo = std::dynamic_pointer_cast<VST3PluginInfo> (get_info ());
+	if (nfo->has_editor.has_value ()) {
+		return nfo->has_editor.value ();
+	}
+
+	bool rv = _plug->has_editor ();
+	nfo->has_editor = rv;
+	return rv;
 }
 
 Steinberg::IPlugView*
@@ -898,7 +907,7 @@ VST3Plugin::do_save_preset (std::string name)
 {
 
 	std::shared_ptr<VST3PluginInfo> nfo = std::dynamic_pointer_cast<VST3PluginInfo> (get_info ());
-	PBD::Searchpath                   psp = nfo->preset_search_path ();
+	PBD::Searchpath                 psp = nfo->preset_search_path ();
 
 	assert (!psp.empty ());
 
@@ -931,7 +940,7 @@ void
 VST3Plugin::do_remove_preset (std::string name)
 {
 	std::shared_ptr<VST3PluginInfo> nfo = std::dynamic_pointer_cast<VST3PluginInfo> (get_info ());
-	PBD::Searchpath                   psp = nfo->preset_search_path ();
+	PBD::Searchpath                 psp = nfo->preset_search_path ();
 
 	assert (!psp.empty ());
 
@@ -1023,7 +1032,7 @@ VST3Plugin::find_presets ()
 
 
 	std::shared_ptr<VST3PluginInfo> info = std::dynamic_pointer_cast<VST3PluginInfo> (get_info ());
-	PBD::Searchpath                   psp  = info->preset_search_path ();
+	PBD::Searchpath                 psp  = info->preset_search_path ();
 
 	std::vector<std::string> preset_files;
 	find_paths_matching_filter (preset_files, psp, vst3_preset_filter, 0, false, true, false);
@@ -1059,7 +1068,7 @@ VST3PluginInfo::load (Session& session)
 		PluginPtr          plugin;
 		Steinberg::VST3PI* plug = new VST3PI (m, unique_id);
 		plugin.reset (new VST3Plugin (session.engine (), session, plug));
-		plugin->set_info (PluginInfoPtr (new VST3PluginInfo (*this)));
+		plugin->set_info (PluginInfoPtr (shared_from_this ()));
 		return plugin;
 	} catch (failed_constructor& err) {
 		;
@@ -1161,6 +1170,10 @@ VST3PI::VST3PI (std::shared_ptr<ARDOUR::VST3PluginModule> m, std::string unique_
 	, _owner (0)
 	, _add_to_selection (false)
 	, _n_factory_presets (0)
+	, _block_rpc (0)
+	, _rpc_queue (RouteProcessorChange::NoProcessorChange, false)
+	, _no_kMono (false)
+	, _restart_component_is_synced (false)
 {
 	using namespace std;
 	IPluginFactory* factory = m->factory ();
@@ -1173,6 +1186,23 @@ VST3PI::VST3PI (std::shared_ptr<ARDOUR::VST3PluginModule> m, std::string unique_
 		throw failed_constructor ();
 	}
 
+	PFactoryInfo fi;
+	if (factory->getFactoryInfo (&fi) == kResultTrue) {
+		/* work around issue with UADx VST3s not recognizing
+		 * Vst::SpeakerArr::kMono. (see commit message for details)
+		 */
+		if (0 == strcmp (fi.vendor, "Universal Audio (UADx)")) {
+			_no_kMono = true;
+		}
+	}
+
+#if !(defined PLATFORM_WINDOWS || defined __APPLE__) /* Linux only */
+	_restart_component_is_synced = m->has_symbol ("yabridge_version");
+	if (_restart_component_is_synced) {
+		DEBUG_TRACE (DEBUG::VST3Config, "VST3PI detected yabridge\n");
+	}
+#endif
+
 #ifndef NDEBUG
 	if (DEBUG_ENABLED (DEBUG::VST3Config)) {
 		char fuid[33];
@@ -1483,7 +1513,7 @@ VST3PI::restartComponent (int32 flags)
 
 	if (flags & Vst::kReloadComponent) {
 		Glib::Threads::Mutex::Lock pl (_process_lock, Glib::Threads::NOT_LOCK);
-		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state) {
+		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state && !_restart_component_is_synced) {
 			pl.acquire ();
 		} else {
 			assert (0); // a plugin should not call this while processing
@@ -1501,7 +1531,7 @@ VST3PI::restartComponent (int32 flags)
 	}
 	if (flags & Vst::kParamValuesChanged) {
 		Glib::Threads::Mutex::Lock pl (_process_lock, Glib::Threads::NOT_LOCK);
-		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state) {
+		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state && !_restart_component_is_synced) {
 			pl.acquire ();
 		}
 		update_shadow_data ();
@@ -1516,7 +1546,7 @@ VST3PI::restartComponent (int32 flags)
 		 * changes are automatically picked up.
 		 */
 		Glib::Threads::Mutex::Lock pl (_process_lock, Glib::Threads::NOT_LOCK);
-		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state) {
+		if (!AudioEngine::instance ()->in_process_thread () && !_is_loading_state && !_restart_component_is_synced) {
 			/* Some plugins (e.g BlendEQ) call this from the process,
 			 * IPlugProcessor::ProcessBuffers. In that case taking the
 			 * _process_lock would deadlock.
@@ -1577,10 +1607,7 @@ VST3PI::restartComponent (int32 flags)
 				p.normal = pi.defaultNormalizedValue;
 			}
 		}
-		Route* r = dynamic_cast<Route*> (_owner);
-		if (r) {
-			r->processors_changed (RouteProcessorChange ()); /* EMIT SIGNAL */
-		}
+		send_processors_changed (RouteProcessorChange ()); /* EMIT SIGNAL */
 	}
 	if (flags & Vst::kIoChanged) {
 		warning << "VST3: Vst::kIoChanged (not implemented)" << endmsg;
@@ -2210,7 +2237,7 @@ VST3PI::enable_io (std::vector<bool> const& ins, std::vector<bool> const& outs)
 		}
 		cnt += n_chn;
 		/* special case for Left only == Mono */
-		if (sa == 1 /*Vst::SpeakerArr::kSpeakerL */) {
+		if (sa == 1 /*Vst::SpeakerArr::kSpeakerL */ && !_no_kMono) {
 			sa = Vst::SpeakerArr::kMono; /* 1 << 19 */
 		}
 
@@ -2234,7 +2261,7 @@ VST3PI::enable_io (std::vector<bool> const& ins, std::vector<bool> const& outs)
 		}
 		cnt += n_chn;
 		/* special case for Left only == Mono */
-		if (sa == 1 /*Vst::SpeakerArr::kSpeakerL */) {
+		if (sa == 1 /*Vst::SpeakerArr::kSpeakerL */ && !_no_kMono) {
 			sa = Vst::SpeakerArr::kMono; /* 1 << 19 */
 		}
 		DEBUG_TRACE (DEBUG::VST3Config, string_compose ("VST3PI::enable_io: activateBus (kAudio, kOutput, %1, %2) used-chn: %3 spk-arr: %4\n", sa_out.size (), enable, _bus_info_out[sa_out.size ()].n_used_chn, std::hex, sa));
@@ -3203,3 +3230,39 @@ VST3PI::resizeView (IPlugView* view, ViewRect* new_size)
 	OnResizeView (new_size->getWidth (), new_size->getHeight ()); /* EMIT SIGNAL */
 	return view->onSize (new_size);
 }
+
+void
+VST3PI::block_notifications ()
+{
+	_block_rpc.fetch_add (1);
+}
+
+void
+VST3PI::resume_notifications ()
+{
+	if (!PBD::atomic_dec_and_test (_block_rpc)) {
+		return;
+	}
+	ARDOUR::RouteProcessorChange rpc (RouteProcessorChange::NoProcessorChange, false);
+	std::swap (rpc, _rpc_queue);
+
+	Route* r = dynamic_cast<Route*> (_owner);
+	if (r && _rpc_queue.type != RouteProcessorChange::NoProcessorChange) {
+		r->processors_changed (rpc); /* EMIT SIGNAL */
+	}
+}
+
+void
+VST3PI::send_processors_changed (RouteProcessorChange const& rpc)
+{
+	if (_block_rpc.load () != 0) {
+		_rpc_queue.type = ARDOUR::RouteProcessorChange::Type (_rpc_queue.type | rpc.type);
+		_rpc_queue.meter_visibly_changed |= rpc.meter_visibly_changed;
+		return;
+	}
+
+	Route* r = dynamic_cast<Route*> (_owner);
+	if (r) {
+		r->processors_changed (rpc); /* EMIT SIGNAL */
+	}
+}
