@@ -16,8 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
-#include <glibmm-2.4/glibmm/main.h>
 #include <boost/optional.hpp>
 
 #include "pbd/abstract_ui.cc" // instantiate template
@@ -26,18 +24,21 @@
 
 #include "ardour/audioengine.h"
 #include "ardour/debug.h"
+#include "ardour/filesystem_paths.h"
 #include "ardour/meter.h"
 #include "ardour/monitor_control.h"
 #include "ardour/phase_control.h"
 #include "ardour/readonly_control.h"
+#include "ardour/route.h"
+#include "ardour/selection.h"
 #include "ardour/session.h"
 #include "ardour/stripable.h"
 #include "ardour/track.h"
 #include "ardour/vca_manager.h"
 
 #include "console1.h"
-#include "c1_gui.h"
 #include "c1_control.h"
+#include "c1_gui.h"
 
 using namespace ARDOUR;
 using namespace ArdourSurface;
@@ -51,6 +52,7 @@ Console1::Console1 (Session& s)
   , blink_state (false)
   , rec_enable_state (false)
 {
+	run_event_loop ();
 	port_setup ();
 }
 
@@ -58,22 +60,30 @@ Console1::~Console1 ()
 {
 	all_lights_out ();
 
+	stop_event_loop ();
 	MIDISurface::drop ();
 
 	tear_down_gui ();
 
-	/* stop event loop */
-	DEBUG_TRACE (DEBUG::Console1, "BaseUI::quit ()\n");
-
-	BaseUI::quit ();
+	for (const auto& b : buttons) {
+		delete b.second;
+	}
+	for (const auto& e : encoders) {
+		delete e.second;
+	}
+	for (const auto& m : meters) {
+		delete m.second;
+	}
+	for (const auto& mb : multi_buttons) {
+		delete mb.second;
+	}
 }
 
-
 void
 Console1::all_lights_out ()
 {
 	for (ButtonMap::iterator b = buttons.begin (); b != buttons.end (); ++b) {
-		b->second.set_led_state (false);
+		b->second->set_led_state (false);
 	}
 }
 
@@ -86,26 +96,8 @@ Console1::set_active (bool yn)
 		return 0;
 	}
 
-	if (yn) {
-
-		/* start event loop */
-
-		DEBUG_TRACE (DEBUG::Console1, "Console1::set_active\n");
-
-		BaseUI::run ();
-
-		connect_session_signals ();
-
-	} else {
-		/* Control Protocol Manager never calls us with false, but
-		 * insteads destroys us.
-		 */
-	}
-
 	ControlProtocol::set_active (yn);
 
-	DEBUG_TRACE (DEBUG::Console1, string_compose ("Console1::set_active done with yn: '%1'\n", yn));
-
 	return 0;
 }
 
@@ -122,6 +114,27 @@ Console1::input_port_name () const
 #endif
 }
 
+XMLNode&
+Console1::get_state () const
+{
+	XMLNode& node = MIDISurface::get_state ();
+	node.set_property ("swap-solo-mute", swap_solo_mute);
+	node.set_property ("create-mapping-stubs", create_mapping_stubs);
+	return node;
+}
+
+int
+Console1::set_state (const XMLNode& node, int version)
+{
+	MIDISurface::set_state (node, version);
+	std::string tmp;
+	node.get_property ("swap-solo-mute", tmp);
+	swap_solo_mute = (tmp == "1");
+	node.get_property ("create-mapping-stubs", tmp);
+	create_mapping_stubs = (tmp == "1");
+	return 0;
+}
+
 std::string
 Console1::output_port_name () const
 {
@@ -135,25 +148,35 @@ Console1::output_port_name () const
 #endif
 }
 
+void
+Console1::run_event_loop ()
+{
+  DEBUG_TRACE (DEBUG::Launchpad, "start event loop\n");
+  BaseUI::run ();
+}
+
+void
+Console1::stop_event_loop ()
+{
+  DEBUG_TRACE (DEBUG::Launchpad, "stop event loop\n");
+  BaseUI::quit ();
+}
+
 int
 Console1::begin_using_device ()
 {
 	DEBUG_TRACE (DEBUG::Console1, "sending device inquiry message...\n");
 
-	if (MIDISurface::begin_using_device ()) {
-		return -1;
-	}
 	/*
 	  with this sysex command we can enter the 'native mode'
 	  But there's no need to do so
 	  f0 7d 20 00 00 00 01 00 7f 49 6f 6c 73 00 f7
 	*/
 
+	load_mappings ();
 	setup_controls ();
 
-	/*
-	Connection to the blink-timer
-	*/
+	/* Connection to the blink-timer */
 	Glib::RefPtr<Glib::TimeoutSource> blink_timeout = Glib::TimeoutSource::create (200); // milliseconds
 	blink_connection = blink_timeout->connect (sigc::mem_fun (*this, &Console1::blinker));
 	blink_timeout->attach (main_loop ()->get_context ());
@@ -162,11 +185,24 @@ Console1::begin_using_device ()
 	Glib::RefPtr<Glib::TimeoutSource> periodic_timer = Glib::TimeoutSource::create (100);
 	periodic_connection = periodic_timer->connect (sigc::mem_fun (*this, &Console1::periodic));
 	periodic_timer->attach (main_loop ()->get_context ());
+	connect_session_signals ();
+	connect_internal_signals ();
+	create_strip_inventory ();
 
 	DEBUG_TRACE (DEBUG::Console1, "************** begin_using_device() ********************\n");
-	create_strip_invetory ();
-	connect_internal_signals ();
-	map_shift (false);
+
+	return MIDISurface::begin_using_device ();
+}
+
+int
+Console1::stop_using_device ()
+{
+	DEBUG_TRACE (DEBUG::Console1, "stop_using_device()\n");
+	blink_connection.disconnect ();
+	periodic_connection.disconnect ();
+	stripable_connections.drop_connections ();
+	console1_connections.drop_connections ();
+	MIDISurface::stop_using_device ();
 	return 0;
 }
 
@@ -176,10 +212,10 @@ Console1::connect_session_signals ()
 	DEBUG_TRACE (DEBUG::Console1, "connect_session_signals\n");
 	// receive routes added
 	session->RouteAdded.connect (
-	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::create_strip_invetory, this), this);
+	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::create_strip_inventory, this), this);
 	// receive VCAs added
 	session->vca_manager ().VCAAdded.connect (
-	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::create_strip_invetory, this), this);
+	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::create_strip_inventory, this), this);
 
 	// receive record state toggled
 	// session->RecordStateChanged.connect(session_connections,
@@ -199,15 +235,22 @@ Console1::connect_session_signals ()
 	// receive rude solo changed
 	session->SoloActive.connect (
 	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::notify_solo_active_changed, this, _1), this);
-	// window.signal_window_state_event().connect (sigc::bind (sigc::mem_fun (*this,
-	// &ARDOUR_UI::tabbed_window_state_event_handler), owner));
+	session->MonitorBusAddedOrRemoved.connect (
+	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::master_monitor_has_changed, this), this);
+	session->MonitorChanged.connect (
+	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::master_monitor_has_changed, this), this);
+	session->RouteAdded.connect (
+	  session_connections, MISSING_INVALIDATOR, boost::bind (&Console1::strip_inventory_changed, this, _1), this);
 }
 
 void
 Console1::connect_internal_signals ()
 {
+	DEBUG_TRACE (DEBUG::Console1, "connect_internal_signals\n");
 	BankChange.connect (console1_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_bank, this), this);
 	ShiftChange.connect (console1_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_shift, this, _1), this);
+	PluginStateChange.connect (
+	  console1_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_plugin_state, this, _1), this);
 	GotoView.connect (
 	  console1_connections,
 	  MISSING_INVALIDATOR,
@@ -224,165 +267,153 @@ Console1::setup_controls ()
 {
 
 	for (uint32_t i = 0; i < 20; ++i) {
-		ControllerButton track_select_button (
-		  *this,
-		  ControllerID (FOCUS1 + i),
-		  boost::function<void (uint32_t)> (boost::bind (&Console1::select, this, i)));
+		new ControllerButton (this,
+		                      ControllerID (FOCUS1 + i),
+		                      boost::function<void (uint32_t)> (boost::bind (&Console1::select, this, i)),
+		                      0,
+		                      boost::function<void (uint32_t)> (boost::bind (&Console1::select_plugin, this, i)));
 	}
 
-	ControllerButton shift_button (
-	  *this, ControllerID::PRESET, boost::function<void (uint32_t)> (boost::bind (&Console1::shift, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::PRESET, boost::function<void (uint32_t)> (boost::bind (&Console1::shift, this, _1)));
 
-	ControllerButton rude_solo (
-	  *this, ControllerID::DISPLAY_ON, boost::function<void (uint32_t)> (boost::bind (&Console1::rude_solo, this, _1)));
-	ControllerButton zoom_button (
-	  *this, ControllerID::MODE, boost::function<void (uint32_t)> (boost::bind (&Console1::zoom, this, _1)));
-	MultiStateButton view (*this,
-	                       ControllerID::EXTERNAL_SIDECHAIN,
-	                       std::vector<uint32_t>{ 0, 63, 127 },
-	                       boost::function<void (uint32_t)> (boost::bind (&Console1::window, this, _1)));
+	new ControllerButton (this,
+	                      ControllerID::TRACK_GROUP,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::plugin_state, this, _1)));
 
-	ControllerButton bank_up_button (
-	  *this, ControllerID::PAGE_UP, boost::function<void (uint32_t)> (boost::bind (&Console1::bank, this, true)));
-	ControllerButton bank_down_button (
-	  *this, ControllerID::PAGE_DOWN, boost::function<void (uint32_t)> (boost::bind (&Console1::bank, this, false)));
+	new ControllerButton (
+	  this, ControllerID::DISPLAY_ON, boost::function<void (uint32_t)> (boost::bind (&Console1::rude_solo, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::MODE, boost::function<void (uint32_t)> (boost::bind (&Console1::zoom, this, _1)));
+	new MultiStateButton (this,
+	                      ControllerID::EXTERNAL_SIDECHAIN,
+	                      std::vector<uint32_t>{ 0, 63, 127 },
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::window, this, _1)));
 
-	ControllerButton mute_button (
-	  *this, ControllerID::MUTE, boost::function<void (uint32_t)> (boost::bind (&Console1::mute, this, _1)));
-	ControllerButton solo_button (
-	  *this, ControllerID::SOLO, boost::function<void (uint32_t)> (boost::bind (&Console1::solo, this, _1)));
-	ControllerButton phase_button (
-	  *this, ControllerID::PHASE_INV, boost::function<void (uint32_t)> (boost::bind (&Console1::phase, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::PAGE_UP, boost::function<void (uint32_t)> (boost::bind (&Console1::bank, this, true)));
+	new ControllerButton (
+	  this, ControllerID::PAGE_DOWN, boost::function<void (uint32_t)> (boost::bind (&Console1::bank, this, false)));
+
+	new ControllerButton (this,
+	                      swap_solo_mute ? ControllerID::SOLO : ControllerID::MUTE,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::mute, this, _1)));
+	new ControllerButton (this,
+	                      swap_solo_mute ? ControllerID::MUTE : ControllerID::SOLO,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::solo, this, _1)));
+
+	new ControllerButton (
+	  this, ControllerID::PHASE_INV, boost::function<void (uint32_t)> (boost::bind (&Console1::phase, this, _1)));
 
 	/*
 	Console 1: Input Gain - Ardour / Mixbus: Trim
 	*/
-	Encoder trim_encoder (
-	  *this, ControllerID::GAIN, boost::function<void (uint32_t)> (boost::bind (&Console1::trim, this, _1)));
+	new Encoder (this, ControllerID::GAIN, boost::function<void (uint32_t)> (boost::bind (&Console1::trim, this, _1)));
 
 	/*
 	Console 1: Volume - Ardour / Mixbus: Gain
 	*/
-	Encoder gain_encoder (
-	  *this, ControllerID::VOLUME, boost::function<void (uint32_t)> (boost::bind (&Console1::gain, this, _1)));
+	new Encoder (
+	  this, ControllerID::VOLUME, boost::function<void (uint32_t)> (boost::bind (&Console1::gain, this, _1)));
 
-	Encoder pan_encoder (
-	  *this, ControllerID::PAN, boost::function<void (uint32_t)> (boost::bind (&Console1::pan, this, _1)));
+	new Encoder (this, ControllerID::PAN, boost::function<void (uint32_t)> (boost::bind (&Console1::pan, this, _1)));
 
 	/* Filter Section*/
-	ControllerButton filter_button (*this,
-	                                ControllerID::FILTER_TO_COMPRESSORS,
-	                                boost::function<void (uint32_t)> (boost::bind (&Console1::filter, this, _1)));
-	Encoder low_cut_encoder (
-	  *this, ControllerID::LOW_CUT, boost::function<void (uint32_t)> (boost::bind (&Console1::low_cut, this, _1)));
-	Encoder high_cut_encoder (
-	  *this, ControllerID::HIGH_CUT, boost::function<void (uint32_t)> (boost::bind (&Console1::high_cut, this, _1)));
+	new ControllerButton (this,
+	                      ControllerID::FILTER_TO_COMPRESSORS,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::filter, this, _1)));
+	new Encoder (
+	  this, ControllerID::LOW_CUT, boost::function<void (uint32_t)> (boost::bind (&Console1::low_cut, this, _1)));
+	new Encoder (
+	  this, ControllerID::HIGH_CUT, boost::function<void (uint32_t)> (boost::bind (&Console1::high_cut, this, _1)));
 
 	/* Gate Section */
-	ControllerButton gate_on_off (
-	  *this, ControllerID::SHAPE, boost::function<void (uint32_t)> (boost::bind (&Console1::gate, this, _1)));
-	ControllerButton gate_scf_listen (
-	  *this,
-	  ControllerID::HARD_GATE,
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::gate_scf, this, _1)),
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::gate_listen, this, _1)));
-	Encoder gate_thresh_encoder (*this,
-	                             ControllerID::SHAPE_GATE,
-	                             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_thresh, this, _1)));
-	Encoder gate_release_encoder (*this,
-	                              ControllerID::SHAPE_RELEASE,
-	                              boost::function<void (uint32_t)> (boost::bind (&Console1::gate_release, this, _1)),
-	                              boost::function<void (uint32_t)> (boost::bind (&Console1::gate_hyst, this, _1)));
-	Encoder gate_attack_encoder (*this,
-	                             ControllerID::SHAPE_SUSTAIN,
-	                             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_attack, this, _1)),
-	                             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_hold, this, _1)));
-	Encoder gate_depth_encoder (*this,
-	                            ControllerID::SHAPE_PUNCH,
-	                            boost::function<void (uint32_t)> (boost::bind (&Console1::gate_depth, this, _1)),
-	                            boost::function<void (uint32_t)> (boost::bind (&Console1::gate_filter_freq, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::SHAPE, boost::function<void (uint32_t)> (boost::bind (&Console1::gate, this, _1)));
+	new ControllerButton (this,
+	                      ControllerID::HARD_GATE,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::gate_scf, this, _1)),
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::gate_listen, this, _1)));
+	new Encoder (this,
+	             ControllerID::SHAPE_GATE,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_thresh, this, _1)));
+	new Encoder (this,
+	             ControllerID::SHAPE_RELEASE,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_release, this, _1)),
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_hyst, this, _1)));
+	new Encoder (this,
+	             ControllerID::SHAPE_SUSTAIN,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_attack, this, _1)),
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_hold, this, _1)));
+	new Encoder (this,
+	             ControllerID::SHAPE_PUNCH,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_depth, this, _1)),
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::gate_filter_freq, this, _1)));
 
-	Meter gate_meter (*this, ControllerID::SHAPE_METER, boost::function<void ()> ([] () {}));
+	new Meter (this, ControllerID::SHAPE_METER, boost::function<void ()> ([] () {}));
 
 	/* EQ Section */
-	ControllerButton eq_on_off (
-	  *this, ControllerID::EQ, boost::function<void (uint32_t)> (boost::bind (&Console1::eq, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::EQ, boost::function<void (uint32_t)> (boost::bind (&Console1::eq, this, _1)));
 
 	for (uint32_t i = 0; i < 4; ++i) {
-		Encoder low_freq_encoder (
-		  *this,
-		  eq_freq_controller_for_band (i),
-		  boost::function<void (uint32_t)> (boost::bind (&Console1::eq_freq, this, i, _1)),
-		  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, i, _1)));
-		Encoder low_gain_encoder (
-		  *this,
-		  eq_gain_controller_for_band (i),
-		  boost::function<void (uint32_t)> (boost::bind (&Console1::eq_gain, this, i, _1)),
-		  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, i + 4, _1)));
+		new Encoder (this,
+		             eq_freq_controller_for_band (i),
+		             boost::function<void (uint32_t)> (boost::bind (&Console1::eq_freq, this, i, _1)),
+		             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, i, _1)));
+		new Encoder (this,
+		             eq_gain_controller_for_band (i),
+		             boost::function<void (uint32_t)> (boost::bind (&Console1::eq_gain, this, i, _1)),
+		             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, i + 4, _1)));
 	}
-	Encoder low_mid_shape_encoder (
-	  *this,
-	  ControllerID::LOW_MID_SHAPE,
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 10, _1)),
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 8, _1)));
-	Encoder high_mid_shape_encoder (
-	  *this,
-	  ControllerID::HIGH_MID_SHAPE,
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 11, _1)),
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 9, _1)));
+	new Encoder (this,
+	             ControllerID::LOW_MID_SHAPE,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 10, _1)),
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 8, _1)));
+	new Encoder (this,
+	             ControllerID::HIGH_MID_SHAPE,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 11, _1)),
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::mb_send_level, this, 9, _1)));
 
-	ControllerButton eq_low_shape (*this,
-	                               ControllerID::LOW_SHAPE,
-	                               boost::function<void (uint32_t)> (boost::bind (&Console1::eq_low_shape, this, _1)));
-	ControllerButton eq_high_shape (
-	  *this,
-	  ControllerID::HIGH_SHAPE,
-	  boost::function<void (uint32_t)> (boost::bind (&Console1::eq_high_shape, this, _1)));
+	new ControllerButton (this,
+	                      ControllerID::LOW_SHAPE,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::eq_low_shape, this, _1)));
+	new ControllerButton (this,
+	                      ControllerID::HIGH_SHAPE,
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::eq_high_shape, this, _1)));
 
-	Encoder drive_encoder (
-	  *this, ControllerID::CHARACTER, boost::function<void (uint32_t)> (boost::bind (&Console1::drive, this, _1)));
+	new Encoder (
+	  this, ControllerID::CHARACTER, boost::function<void (uint32_t)> (boost::bind (&Console1::drive, this, _1)));
 
 	/* Compressor Section */
-	ControllerButton comp_on_off (
-	  *this, ControllerID::COMP, boost::function<void (uint32_t)> (boost::bind (&Console1::comp, this, _1)));
-	MultiStateButton comp_mode (*this,
-	                            ControllerID::ORDER,
-	                            std::vector<uint32_t>{ 0, 63, 127 },
-	                            boost::function<void (uint32_t)> (boost::bind (&Console1::comp_mode, this, _1)));
+	new ControllerButton (
+	  this, ControllerID::COMP, boost::function<void (uint32_t)> (boost::bind (&Console1::comp, this, _1)));
+	new MultiStateButton (this,
+	                      ControllerID::ORDER,
+	                      std::vector<uint32_t>{ 0, 63, 127 },
+	                      boost::function<void (uint32_t)> (boost::bind (&Console1::comp_mode, this, _1)));
 
-	Encoder comp_thresh_encoder (*this,
-	                             ControllerID::COMP_THRESH,
-	                             boost::function<void (uint32_t)> (boost::bind (&Console1::comp_thresh, this, _1)));
-	Encoder comp_attack_encoder (*this,
-	                             ControllerID::COMP_ATTACK,
-	                             boost::function<void (uint32_t)> (boost::bind (&Console1::comp_attack, this, _1)));
-	Encoder comp_release_encoder (*this,
-	                              ControllerID::COMP_RELEASE,
-	                              boost::function<void (uint32_t)> (boost::bind (&Console1::comp_release, this, _1)));
-	Encoder comp_ratio_encoder (*this,
-	                            ControllerID::COMP_RATIO,
-	                            boost::function<void (uint32_t)> (boost::bind (&Console1::comp_ratio, this, _1)));
-	Encoder comp_makeup_encoder (
-	  *this, ControllerID::COMP_PAR, boost::function<void (uint32_t)> (boost::bind (&Console1::comp_makeup, this, _1)));
-	Encoder comp_emph_encoder (
-	  *this, ControllerID::DRIVE, boost::function<void (uint32_t)> (boost::bind (&Console1::comp_emph, this, _1)));
+	new Encoder (this,
+	             ControllerID::COMP_THRESH,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::comp_thresh, this, _1)));
+	new Encoder (this,
+	             ControllerID::COMP_ATTACK,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::comp_attack, this, _1)));
+	new Encoder (this,
+	             ControllerID::COMP_RELEASE,
+	             boost::function<void (uint32_t)> (boost::bind (&Console1::comp_release, this, _1)));
+	new Encoder (
+	  this, ControllerID::COMP_RATIO, boost::function<void (uint32_t)> (boost::bind (&Console1::comp_ratio, this, _1)));
+	new Encoder (
+	  this, ControllerID::COMP_PAR, boost::function<void (uint32_t)> (boost::bind (&Console1::comp_makeup, this, _1)));
+	new Encoder (
+	  this, ControllerID::DRIVE, boost::function<void (uint32_t)> (boost::bind (&Console1::comp_emph, this, _1)));
 
-	Meter compressor_meter (*this, ControllerID::COMP_METER, boost::function<void ()> ([] () {}));
+	new Meter (this, ControllerID::COMP_METER, boost::function<void ()> ([] () {}));
 
 	/* Output Section */
-	Meter output_meter_l (*this, ControllerID::OUTPUT_METER_L, boost::function<void ()> ([] () {}));
-	Meter output_meter_r (*this, ControllerID::OUTPUT_METER_R, boost::function<void ()> ([] () {}));
-}
-
-int
-Console1::stop_using_device ()
-{
-	DEBUG_TRACE (DEBUG::Console1, "stop_using_device()\n");
-
-	blink_connection.disconnect ();
-	periodic_connection.disconnect ();
-	stripable_connections.drop_connections ();
-	return 0;
+	new Meter (this, ControllerID::OUTPUT_METER_L, boost::function<void ()> ([] () {}));
+	new Meter (this, ControllerID::OUTPUT_METER_R, boost::function<void ()> ([] () {}));
 }
 
 void
@@ -394,14 +425,16 @@ Console1::handle_midi_controller_message (MIDI::Parser&, MIDI::EventTwoBytes* tb
 	DEBUG_TRACE (DEBUG::Console1,
 	             string_compose ("handle_midi_controller_message cn: '%1' val: '%2'\n", controller_number, value));
 	try {
-		Encoder e = get_encoder (ControllerID (controller_number));
-		if (shift_state && e.shift_action) {
-			e.shift_action (value);
+		Encoder* e = get_encoder (ControllerID (controller_number));
+		if (in_plugin_state && e->plugin_action) {
+			e->plugin_action (value);
+		} else if (shift_state && e->shift_action) {
+			e->shift_action (value);
 		} else {
-			e.action (value);
+			e->action (value);
 		}
 		return;
-	} catch (ControlNotFoundException& e) {
+	} catch (ControlNotFoundException const&) {
 		DEBUG_TRACE (DEBUG::Console1,
 		             string_compose ("handle_midi_controller_message: encoder not found cn: "
 		                             "'%1' val: '%2'\n",
@@ -410,16 +443,19 @@ Console1::handle_midi_controller_message (MIDI::Parser&, MIDI::EventTwoBytes* tb
 	}
 
 	try {
-		ControllerButton& b = get_button (ControllerID (controller_number));
-		if (shift_state && b.shift_action) {
+		ControllerButton* b = get_button (ControllerID (controller_number));
+		if (in_plugin_state && b->plugin_action) {
+			DEBUG_TRACE (DEBUG::Console1, "Executing plugin_action\n");
+			b->plugin_action (value);
+		} else if (shift_state && b->shift_action) {
 			DEBUG_TRACE (DEBUG::Console1, "Executing shift_action\n");
-			b.shift_action (value);
+			b->shift_action (value);
 		} else {
 			DEBUG_TRACE (DEBUG::Console1, "Executing action\n");
-			b.action (value);
+			b->action (value);
 		}
 		return;
-	} catch (ControlNotFoundException& e) {
+	} catch (ControlNotFoundException const&) {
 		DEBUG_TRACE (DEBUG::Console1,
 		             string_compose ("handle_midi_controller_message: button not found cn: "
 		                             "'%1' val: '%2'\n",
@@ -428,15 +464,15 @@ Console1::handle_midi_controller_message (MIDI::Parser&, MIDI::EventTwoBytes* tb
 	}
 
 	try {
-		MultiStateButton mb = get_mbutton (ControllerID (controller_number));
-		if (shift_state && mb.shift_action) {
-			mb.shift_action (value);
+		MultiStateButton* mb = get_mbutton (ControllerID (controller_number));
+		if (shift_state && mb->shift_action) {
+			mb->shift_action (value);
 		} else {
-			mb.action (value);
+			mb->action (value);
 		}
 
 		return;
-	} catch (ControlNotFoundException& e) {
+	} catch (ControlNotFoundException const&) {
 		DEBUG_TRACE (DEBUG::Console1,
 		             string_compose ("handle_midi_controller_message: mbutton not found cn: "
 		                             "'%1' val: '%2'\n",
@@ -445,19 +481,13 @@ Console1::handle_midi_controller_message (MIDI::Parser&, MIDI::EventTwoBytes* tb
 	}
 }
 
-void
-Console1::tabbed_window_state_event_handler (GdkEventWindowState* ev, void* object)
-{
-	DEBUG_TRACE (DEBUG::Console1, string_compose ("tabbed_window_state_event_handler: %1\n", ev->type));
-}
-
 void
 Console1::notify_solo_active_changed (bool state)
 {
 	DEBUG_TRACE (DEBUG::Console1, "notify_active_solo_changed() \n");
 	try {
-		get_button (ControllerID::DISPLAY_ON).set_led_value (state ? 127 : 0);
-	} catch (ControlNotFoundException& e) {
+		get_button (ControllerID::DISPLAY_ON)->set_led_value (state ? 127 : 0);
+	} catch (ControlNotFoundException const&) {
 		DEBUG_TRACE (DEBUG::Console1, "button not found");
 	}
 }
@@ -479,18 +509,27 @@ void
 Console1::stripable_selection_changed ()
 {
 	DEBUG_TRACE (DEBUG::Console1, "stripable_selection_changed \n");
-	set_current_stripable (first_selected_stripable ());
+	if (!_in_use)
+		return;
+
+	std::shared_ptr<Stripable> r = ControlProtocol::first_selected_stripable ();
+	if (r) {
+		set_current_stripable (r);
+	}
 }
 
 void
 Console1::drop_current_stripable ()
 {
+	DEBUG_TRACE (DEBUG::Console1, "drop_current_stripable \n");
 	if (_current_stripable) {
 		if (_current_stripable == session->monitor_out ()) {
 			set_current_stripable (session->master_out ());
 		} else {
-			set_current_stripable (std::shared_ptr<Stripable> ());
+			set_current_stripable (_current_stripable);
 		}
+	} else {
+		set_current_stripable (std::shared_ptr<Stripable> ());
 	}
 }
 
@@ -505,6 +544,8 @@ Console1::set_current_stripable (std::shared_ptr<Stripable> r)
 	if (_current_stripable) {
 		DEBUG_TRACE (DEBUG::Console1, "current_stripable found:  \n");
 
+		current_plugin_index = -1;
+
 		PresentationInfo pi = _current_stripable->presentation_info ();
 
 		DEBUG_TRACE (DEBUG::Console1, string_compose ("current_stripable %1 - %2\n", pi.order (), pi.flags ()));
@@ -536,14 +577,20 @@ Console1::set_current_stripable (std::shared_ptr<Stripable> r)
 		_current_stripable->DropReferences.connect (
 		  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::drop_current_stripable, this), this);
 
-		_current_stripable->mute_control ()->Changed.connect (
-		  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_mute, this), this);
+		if (_current_stripable->mute_control ()) {
+			_current_stripable->mute_control ()->Changed.connect (
+			  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_mute, this), this);
+		}
 
-		_current_stripable->solo_control ()->Changed.connect (
-		  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_solo, this), this);
+		if (_current_stripable->solo_control ()) {
+			_current_stripable->solo_control ()->Changed.connect (
+			  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_solo, this), this);
+		}
 
-		_current_stripable->phase_control ()->Changed.connect (
-		  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_phase, this), this);
+		if (_current_stripable->phase_control ()) {
+			_current_stripable->phase_control ()->Changed.connect (
+			  stripable_connections, MISSING_INVALIDATOR, boost::bind (&Console1::map_phase, this), this);
+		}
 
 		// Rec Enabled
 		std::shared_ptr<Track> t = std::dynamic_pointer_cast<Track> (_current_stripable);
@@ -745,7 +792,7 @@ Console1::set_current_stripable (std::shared_ptr<Stripable> r)
 	}
 
 	// ToDo: subscribe to the fader automation modes so we can light the LEDs
-
+	map_shift (shift_state);
 	map_stripable_state ();
 }
 
@@ -821,15 +868,23 @@ Console1::map_stripable_state ()
 void
 Console1::stop_blinking (ControllerID id)
 {
-	blinkers.remove (id);
-	get_button (id).set_led_state (false);
+	try {
+		blinkers.remove (id);
+		get_button (id)->set_led_state (false);
+	} catch (ControlNotFoundException const&) {
+		DEBUG_TRACE (DEBUG::Console1, "Button to stop blinking not found ...\n");
+	}
 }
 
 void
 Console1::start_blinking (ControllerID id)
 {
-	blinkers.push_back (id);
-	get_button (id).set_led_state (true);
+	try {
+		blinkers.push_back (id);
+		get_button (id)->set_led_state (true);
+	} catch (ControlNotFoundException const&) {
+		DEBUG_TRACE (DEBUG::Console1, "Button to start blinking not found ...\n");
+	}
 }
 
 bool
@@ -839,8 +894,8 @@ Console1::blinker ()
 
 	for (Blinkers::iterator b = blinkers.begin (); b != blinkers.end (); b++) {
 		try {
-			get_button (*b).set_led_state (blink_state);
-		} catch (ControlNotFoundException& e) {
+			get_button (*b)->set_led_state (blink_state);
+		} catch (ControlNotFoundException const&) {
 			DEBUG_TRACE (DEBUG::Console1, "Blinking Button not found ...\n");
 		}
 	}
@@ -848,40 +903,40 @@ Console1::blinker ()
 	return true;
 }
 
-ControllerButton&
+ControllerButton*
 Console1::get_button (ControllerID id) const
 {
 	ButtonMap::const_iterator b = buttons.find (id);
 	if (b == buttons.end ())
 		throw (ControlNotFoundException ());
-	return const_cast<ControllerButton&> (b->second);
+	return const_cast<ControllerButton*> (b->second);
 }
 
-Meter&
+Meter*
 Console1::get_meter (ControllerID id) const
 {
 	MeterMap::const_iterator m = meters.find (id);
 	if (m == meters.end ())
 		throw (ControlNotFoundException ());
-	return const_cast<Meter&> (m->second);
+	return const_cast<Meter*> (m->second);
 }
 
-Encoder&
+Encoder*
 Console1::get_encoder (ControllerID id) const
 {
 	EncoderMap::const_iterator m = encoders.find (id);
 	if (m == encoders.end ())
 		throw (ControlNotFoundException ());
-	return const_cast<Encoder&> (m->second);
+	return const_cast<Encoder*> (m->second);
 }
 
-MultiStateButton&
+MultiStateButton*
 Console1::get_mbutton (ControllerID id) const
 {
 	MultiStateButtonMap::const_iterator m = multi_buttons.find (id);
 	if (m == multi_buttons.end ())
 		throw (ControlNotFoundException ());
-	return const_cast<MultiStateButton&> (m->second);
+	return const_cast<MultiStateButton*> (m->second);
 }
 
 ControllerID
@@ -921,14 +976,14 @@ Console1::periodic_update_meter ()
 			}
 			try {
 				if (val_l != last_output_meter_l) {
-					get_meter (OUTPUT_METER_L).set_value (val_l);
+					get_meter (OUTPUT_METER_L)->set_value (val_l);
 					last_output_meter_l = val_l;
 				}
 				if (val_r != last_output_meter_r) {
-					get_meter (OUTPUT_METER_R).set_value (val_r);
+					get_meter (OUTPUT_METER_R)->set_value (val_r);
 					last_output_meter_r = val_r;
 				}
-			} catch (ControlNotFoundException& e) {
+			} catch (ControlNotFoundException const&) {
 				DEBUG_TRACE (DEBUG::Console1, "Meter not found ...\n");
 			}
 		}
@@ -942,10 +997,10 @@ Console1::periodic_update_meter ()
 			}
 			try {
 				if (val != last_gate_meter) {
-					get_meter (SHAPE_METER).set_value (val);
+					get_meter (SHAPE_METER)->set_value (val);
 					last_gate_meter = val;
 				}
-			} catch (ControlNotFoundException& e) {
+			} catch (ControlNotFoundException const&) {
 				DEBUG_TRACE (DEBUG::Console1, "Meter not found ...\n");
 			}
 		}
@@ -962,9 +1017,9 @@ Console1::periodic_update_meter ()
 				if (val != last_comp_redux) {
 					last_comp_redux = val;
 					val = val * 0.6 + last_comp_redux * 0.4;
-					get_meter (COMP_METER).set_value (val);
+					get_meter (COMP_METER)->set_value (val);
 				}
-			} catch (ControlNotFoundException& e) {
+			} catch (ControlNotFoundException const&) {
 				DEBUG_TRACE (DEBUG::Console1, "Meter not found ...\n");
 			}
 		}
@@ -1051,16 +1106,16 @@ Console1::midi_to_control (Controllable controllable, uint32_t val, uint32_t max
 }
 
 void
-Console1::create_strip_invetory ()
+Console1::create_strip_inventory ()
 {
-	DEBUG_TRACE (DEBUG::Console1, "create_strip_invetory()\n");
-	StripableList sl;
+	DEBUG_TRACE (DEBUG::Console1, "create_strip_inventory()\n");
 	boost::optional<order_t> master_order;
 	strip_inventory.clear ();
-	session->get_stripables (sl);
+	StripableList sl = session->get_stripables ();
 	uint32_t index = 0;
 	for (const auto& s : sl) {
 		PresentationInfo pi = s->presentation_info ();
+		DEBUG_TRACE (DEBUG::Console1, string_compose ("%1: ", s->name ()));
 		if (pi.flags () & ARDOUR::PresentationInfo::Hidden) {
 			DEBUG_TRACE (DEBUG::Console1, string_compose ("strip hidden: index %1, order %2\n", index, pi.order ()));
 			continue;
@@ -1073,7 +1128,12 @@ Console1::create_strip_invetory ()
 		}
 		if (pi.flags () & ARDOUR::PresentationInfo::MonitorOut) {
 			DEBUG_TRACE (DEBUG::Console1,
-			             string_compose ("monitor strip found at index %1, order %2\n", index, pi.order ()));
+			             string_compose ("monitor strip found at index %1, order %2 - ignoring\n", index, pi.order ()));
+			continue;
+		}
+		if (pi.flags () & ARDOUR::PresentationInfo::FoldbackBus) {
+			DEBUG_TRACE (DEBUG::Console1,
+			             string_compose ("foldback bus found at index %1, order %2\n", index, pi.order ()));
 			continue;
 		}
 		strip_inventory.insert (std::make_pair (index, pi.order ()));
@@ -1081,10 +1141,12 @@ Console1::create_strip_invetory ()
 		++index;
 	}
 	if (master_order) {
+		master_index = index;
 		strip_inventory.insert (std::make_pair (index, master_order.value ()));
 	}
+	max_strip_index = index;
 	DEBUG_TRACE (DEBUG::Console1,
-	             string_compose ("create_strip_invetory - inventory size %1\n", strip_inventory.size ()));
+	             string_compose ("create_strip_inventory - inventory size %1\n", strip_inventory.size ()));
 }
 
 order_t
@@ -1110,9 +1172,40 @@ Console1::get_index_by_inventory_order (order_t order)
 void
 Console1::select_rid_by_index (uint32_t index)
 {
+	bool success = true;
+	DEBUG_TRACE (DEBUG::Console1, "select_rid_by_index()\n");
+	int offset = session->monitor_out () ? 1 : 0;
+	DEBUG_TRACE (DEBUG::Console1, string_compose ("offset %1\n", offset));
+	uint_fast32_t rid = 0;
 #ifdef MIXBUS
-	set_rid_selection (index + 1);
+	rid = index + offset;
 #else
-	set_rid_selection (index + 2);
+	if (index == master_index) {
+		rid = 1;
+	} else {
+		rid = index + 1 + offset;
+	}
 #endif
+	DEBUG_TRACE (DEBUG::Console1, string_compose ("rid %1\n", rid));
+	if (rid > ( max_strip_index + 1 + offset )) {
+		success =  false;
+	}
+	std::shared_ptr<Stripable> s = session->get_remote_nth_stripable (rid, PresentationInfo::MixerStripables);
+	if (s) {
+		session->selection ().select_stripable_and_maybe_group (s, true, false, 0);
+	} else {
+		success = false;
+	}
+	if (!success) {
+		map_select ();
+	}
+}
+
+void
+Console1::master_monitor_has_changed ()
+{
+	DEBUG_TRACE (DEBUG::Console1, "master_monitor_has_changed()\n");
+	bool monitor_active = session->monitor_active ();
+	DEBUG_TRACE (DEBUG::Console1, string_compose ("master_monitor_has_changed - monitor active %1\n", monitor_active));
+	create_strip_inventory ();
 }
